/*
 * lcd.c
 */

#include "lcd.h"
#include "gpio.h"

void lcd_send_command(struct lcdmodule module, unsigned char cmd)
{
	GPIO_CLR = 1<<module.en; //Nollaa RS ja EN pinnit
	GPIO_CLR = 1<<module.rs;

	GPIO_CLR = LCD_CMD_LOWBITS(0xF); //Nollaa GPIO datapinnit

	GPIO_SET = LCD_CMD_HIGHBITS(cmd); //Kirjoita komennon 4 ylinta bittia datalinjaan
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(41);

	GPIO_CLR = LCD_CMD_LOWBITS(0xF); //Nollaa GPIO datapinnit

	GPIO_SET = LCD_CMD_LOWBITS(cmd); //Kirjoita komennon 4 alinta bittia datalinjaan
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(41);
}

void lcd_send_data(struct lcdmodule module, unsigned char cmd)
{
	GPIO_CLR = 1<<module.en; //Nollaa EN pinni
	GPIO_SET = 1<<module.rs; //Laita RS pinni ykkostilaan silla lahetetaan dataa

	GPIO_CLR = LCD_CMD_LOWBITS(0xF); //Nollaa GPIO datapinnit

	GPIO_SET = LCD_CMD_HIGHBITS(cmd); //Kirjoita datan 4 ylinta bittia datalinjaan
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(50);

	GPIO_CLR = LCD_CMD_LOWBITS(0xF); //Nollaa GPIO datapinnit

	GPIO_SET = LCD_CMD_LOWBITS(cmd); //Kirjoita datan 4 alinta bittia datalinjaan
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(50);
}

struct lcdmodule lcd_init(unsigned char en, unsigned char rs, unsigned char d4, unsigned char d5, unsigned char d6, unsigned char d7)
{
	struct lcdmodule module;

	module.en=en;
	module.rs=rs;

	module.datapins[0]=d4;
	module.datapins[1]=d5;
	module.datapins[2]=d6;
	module.datapins[3]=d7;

	setup_io();

	// Aseta GPIO porttien suunnat
	INP_GPIO(module.en); // Taytyy kutsua INP_GPIO ennen OUT_GPIO:n kayttoa
	INP_GPIO(module.rs);
	OUT_GPIO(module.en);
	OUT_GPIO(module.rs);

	int i;
	for(i=0; i<=3; i++)
	{
		INP_GPIO(module.datapins[i]);
		OUT_GPIO(module.datapins[i]);
	}

	// Alusta naytto aluksi 8-bit tilaan
	GPIO_CLR = 1<<module.en;
	GPIO_CLR = 1<<module.rs;

	GPIO_CLR = LCD_CMD_LOWBITS(0xF);
	GPIO_SET = LCD_CMD_HIGHBITS(LCD_FUNCTIONSET | LCD_8BITMODE); //8-bit tilaan
	GPIO_SET = 1<<module.en; // Komento kirjoitetaan 3 kertaa
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(4200);
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(150);
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(150);

	// Lahetetaan uusi komento, joka laittaa nayton 4-bit tilaan
	GPIO_CLR = LCD_CMD_LOWBITS(0xF);
	GPIO_SET = LCD_CMD_HIGHBITS(LCD_FUNCTIONSET | LCD_4BITMODE);
	GPIO_SET = 1<<module.en;
	usleep(1);
	GPIO_CLR = 1<<module.en;
	usleep(41);

	// Nyt naytto toimii 4-bit tilassa joten sille voi lahettaa komentoja
	// normaaleilla funktioilla.

	lcd_send_command(module, LCD_FUNCTIONSET | LCD_2LINE); // Laita 2-rivinen tila
	lcd_send_command(module, LCD_DISPLAYCONTROL | LCD_DISPLAYON | LCD_BLINKOFF); // Naytto ja kursori paalle, kursori vilkkumaan
	lcd_send_command(module, LCD_CLEARDISPLAY); // Tyhjenna naytto
	usleep(1600); // Komento voi kestaa kauemmin

	lcd_send_command(module, LCD_ENTRYMODESET | LCD_ENTRYLEFT);
	lcd_send_command(module, LCD_SETDDRAMADDR | 0x0); // Aseta nayton osoite
	return module;
}

void lcd_clear(struct lcdmodule module)
{
	lcd_send_command(module, LCD_CLEARDISPLAY);
	usleep(1600);
}

void lcd_print(struct lcdmodule module, char *s)
{
	while(*s)
	{
		// TODO: Muokkaa koodi toimimaan nayttojen kanssa jossa on enemman kuin 2 rivia
		if(*s=='\n')
		{
			lcd_goto_xy(module, 0, 1);
		}else{
			lcd_send_data(module, *s);
		}
		s++;
	}
}

void lcd_goto_xy(struct lcdmodule module, unsigned char x, unsigned char y)
{
	int lcd_offsets[] = {0x0, 0x40, 0x20, 0x60};
	unsigned char pos;

	if(y >= 4)
		y=3;

	pos=lcd_offsets[y] + x;
	lcd_send_command(module, LCD_SETDDRAMADDR | pos);
}
