/* lcdd.c
 * LCD daemon for raspberrypi
 * GPIO LCD screen.
 */

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <ifaddrs.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <signal.h>
#include <sys/sysinfo.h>

#include "lcd.h"
#include "gpio.h"

#define SECS_IN_DAY 86400
#define SECS_IN_HOUR 3600
#define SECS_IN_MIN 60

struct ifaddrs *ifaddr, *ifa;
struct lcdmodule module;
struct sysinfo info;

char info_to_print[32];

void cleanup();
void lcd_print_interface_info(struct lcdmodule module, const char *interface_name);
void lcd_print_sysinfo();

int main(int argc, char **argv) {
	int family;

	//Rekisteroi signaalit jotta naytto tyhjennetaan kun ohjelma suljetaan
	signal(SIGINT, &cleanup);
	signal(SIGTERM, &cleanup);
	
	module=lcd_init(17, 18, 22, 23, 24, 25);
//	lcd_send_command(module, LCD_DISPLAYCONTROL | LCD_BLINKON | LCD_CURSORON);

	while(1)
	{
		if(getifaddrs(&ifaddr) !=0)
		{
			perror("getifaddrs");
		}

		for(ifa=ifaddr; ifa != NULL; ifa=ifa->ifa_next)
		{
			if(ifa->ifa_addr==NULL)
				continue;

			family=ifa->ifa_addr->sa_family;
			if(family != AF_INET)
				continue;

			// Tulosta interfacen nimi ja tiedot naytolle
			sprintf(info_to_print, "%s:\n%s", ifa->ifa_name,
				inet_ntoa(((struct sockaddr_in *)ifa->ifa_addr)->sin_addr)); // Kokoa tiedot jotka tulostetaan naytolle
			lcd_print(module, info_to_print);
			sleep(2);
			lcd_clear(module);
		}

		lcd_print_sysinfo();
	}
	return 0;
}

void cleanup()
{
	freeifaddrs(ifaddr);
	lcd_clear(module);
	exit(0);
}

void lcd_print_sysinfo()
{
	sysinfo(&info);

	long days;
	long hours;
	long minutes;
	long seconds;
	long uptime = info.uptime;

	days = uptime / SECS_IN_DAY;
	uptime -= days * SECS_IN_DAY;

	hours = uptime / SECS_IN_HOUR;
	uptime -= hours * SECS_IN_HOUR;

	minutes = uptime / SECS_IN_MIN;
	uptime -= minutes * SECS_IN_MIN;

	seconds=uptime;

	sprintf(info_to_print, "uptime:\n%dd%ih%im%ds", days, hours, minutes, seconds);
	lcd_print(module, info_to_print);
	sleep(2);
	lcd_clear(module);

	sprintf(info_to_print, "load average:\n%0.2f %0.2f %0.2f", (float)info.loads[0] / 65536.0,
		(float)info.loads[1] / 65536.0, (float)info.loads[2] / 65536.0);
	lcd_print(module, info_to_print);
	sleep(2);	
	lcd_clear(module);
}
